@@ -1,15 +1,19 @@
 module Main.Gens where
 
-import Main.Prelude hiding (assert, isRight, isLeft)
-import Test.QuickCheck
+import Main.Prelude hiding (assert, isRight, isLeft, choose)
+import Test.QuickCheck hiding (vector)
 import Test.QuickCheck.Instances
+import JSONAST (JSON)
 import qualified Main.PTI as PTI
 import qualified Data.Scientific as Scientific
 import qualified Data.UUID as UUID
 import qualified Data.Vector as Vector
+import qualified Rebase.Data.HashMap.Strict as HashMap
 import qualified PostgreSQL.Binary.Data as Data
 import qualified PostgreSQL.Binary.Encoder as Encoder
 import qualified Data.Text as Text
+import qualified Data.Aeson as Aeson
+import qualified JSONAST
 
 
 -- * Generators
@@ -19,6 +23,53 @@ auto :: Arbitrary a => Gen a
 auto =
   arbitrary
 
+json :: Gen JSON
+json =
+  byDepth 0
+  where
+    byDepth depth =
+      frequency (primitives <> composites)
+      where
+        primitives =
+          map (freq,) [null, bool, number, string]
+          where
+            freq =
+              maxFreq
+        composites =
+          map (freq,) [array, object]
+          where
+            freq =
+              maxFreq - depth
+        maxFreq =
+          6
+        null =
+          pure JSONAST.Null
+        bool =
+          fmap JSONAST.Bool arbitrary
+        number =
+          fmap JSONAST.Number arbitrary
+        string =
+          fmap JSONAST.String text
+        array =
+          fmap JSONAST.Array (vector (byDepth (succ depth)))
+        object =
+          fmap JSONAST.Object (hashMap text (byDepth (succ depth)))
+
+vector :: Gen a -> Gen (Vector a)
+vector element =
+  join $ Vector.replicateM <$> arbitrary <*> pure element
+
+hashMap :: (Eq a, Hashable a) => Gen a -> Gen b -> Gen (HashMap a b)
+hashMap key value =
+  fmap HashMap.fromList $ join $ replicateM <$> arbitrary <*> pure row
+  where
+    row =
+      (,) <$> key <*> value
+
+aeson :: Gen Aeson.Value
+aeson =
+  fmap unsafeCoerce json
+
 postgresInt :: (Bounded a, Ord a, Integral a, Arbitrary a) => Gen a
 postgresInt =
   arbitrary >>= \x -> if x > halfMaxBound then postgresInt else pure x
